CF=DRotateToMinimal;

Needs["LinAlg`"]
Print["Loading ChordDiagrams..."]

SetAttributes[Diagram, Orderless]
Legs[DCut]=1;		Deg[DCut]=0;
Legs[m_. * Chord]=2*m;	Deg[m_. * Chord]=m;	SetAttributes[Chord, Orderless];
Legs[m_. * Arrow]=2*m;	Deg[m_. * Arrow]=m;
Legs[D4T]=3;		Deg[D4T]=2;	D4T[i_, j_, k_] := D4T[i, k, j] /; j>k
Legs[D6T]=3;		Deg[D6T]=2;
Legs[DF]=1;		Deg[DF]=1;
Legs[DFL]=1;		Deg[DFL]=1;
Legs[DFR]=1;		Deg[DFR]=1;
Legs[D1T]=1;		Deg[D1T]=2;
Legs[D2T]=1;		Deg[D2T]=3;
Legs[D3T]=3;		Deg[D3T]=2;
Legs[D8T1]=6;		Deg[D8T1]=4;
Legs[D8T2]=4;		Deg[D8T2]=3;
Legs[D14T]=4;		Deg[D14T]=3;
Legs[DG]=1;		Deg[DG]=1;
Legs[DFF]=2;		Deg[DFF]=2;
Legs[DFS]=2;		Deg[DFS]=2;
Legs[DF4T]=4;		Deg[DF4T]=3;

Legs[0]=0
Legs[specs_Plus] := Legs /@ specs

Deg[0]=0
Deg[specs_Plus] := Deg /@ specs

DRotateLeft[d_Diagram] := Module[
  {labels=Union @@ (List @@ Apply[List, d, {1}])},
  d /. Thread[Rule[labels, RotateLeft[labels]]]
]

DRotateToMinimal[expr_] := expr /. d_Diagram :> Module[
  {bd=d,rd=DRotateLeft[d]},
  While[rd=!=d,
    bd=First[Sort[{bd,rd}]];
    rd=DRotateLeft[rd]
  ];
  bd
]

If[Head[$DefaultRelations] =!= List,
  (* $DefaultRelations = {D1T, D2T, D3T, D8T1, D8T2, D14T, DG} *)
  $DefaultRelations = {D3T, D8T1, D8T2, D14T}
]

Place[0,places_List] = {Diagram[]}

Place[m_. * Chord, places_List] := Module[
  {l={}, n=Length[places], i1, i2},
  For[i1=1, i1<=n-2m+1, ++i1, For[i2=i1+1, i2<=n, ++i2,
    l=Join[l,
      Append[#, Chord[places[[i1]], places[[i2]]]]&
        /@ Place[(m-1)*Chord, Drop[Delete[places,i2],i1]]
    ]
  ]]; l
]

Place[m_. * Arrow, places_List] := Module[
  {l={}, n=Length[places], i1, i2},
  For[i1=1, i1<=n-2m+1, ++i1, For[i2=i1+1, i2<=n, ++i2,
    l=Join[l,
      Append[#, Arrow[places[[i1]], places[[i2]]]]&
        /@ Place[(m-1)*Arrow, Drop[Delete[places,i2],i1]],
      Append[#, Arrow[places[[i2]], places[[i1]]]]&
        /@ Place[(m-1)*Arrow, Drop[Delete[places,i2],i1]]
    ]
  ]]; l
]

Place[D4T, places_List] := Module[
  {l={}, n=Length[places], t1, t2, t3},
  For[t1=1, t1<=n, ++t1,
    For[t2=1, t2<n, ++t2,
      If[t2!=t1, For[t3=t2+1, t3<=n, ++t3,
        If[t3!=t1,
          AppendTo[l, Diagram[D4T @@ places[[{t1,t2,t3}]]]]
        ]
      ]]
    ]
  ];
  l
]
Place[DF, places_List] := Diagram /@ DF /@ places
Place[DFL, places_List] := Diagram /@ DFL /@ places
Place[DFR, places_List] := Diagram /@ DFR /@ places

Place[D6T, places_List] := Module[
  {l={}, n=Length[places], t1, t2, t3},
  For[t1=1, t1<=n, ++t1,
    For[t2=1, t2<=n, ++t2,
      If[t2!=t1, For[t3=1, t3<=n, ++t3,
        If[t3!=t1 && t3!=t2,
          AppendTo[l, Diagram[D6T @@ places[[{t1,t2,t3}]]]]
        ]
      ]]
    ]
  ];
  l
]

Place[DCut, places_List] := Diagram /@ DCut /@ places
Place[D1T, places_List] := Diagram /@ D1T /@ places
Place[D2T, places_List] := Diagram /@ D2T /@ places
Place[D3T, places_List] := Flatten[Table[
  Diagram[D3T @@ places[[{t1, t2, t3}]]],
  {t3, 3, Length[places]}, {t2, 2, t3-1}, {t1, t2-1}
]]
Place[D8T1, places_List] := Diagram /@ Union[Place[{D4T, D4T}, places]] /. {
  Diagram -> Join, D4T -> D8T1
}
Place[D8T2, places_List] := Diagram /@ Union[Place[2*Chord, places]] /. {
  Diagram -> Join, Chord -> D8T2
}
Place[D14T, places_List] := Module[
  {l={}, n=Length[places], t1, t2, t3, t4},
  For[t1=1, t1<=n, ++t1,
    For[t2=1, t2<=n, ++t2, If[t2!=t1,
      For[t3=1, t3<n, ++t3, If[t3!=t1 && t3!=t2,
        For[t4=t3+1, t4<=n, ++t4, If[t4!=t1 && t4!=t2,
          AppendTo[l, Diagram[D14T @@ places[[{t1,t2,t3,t4}]]]]
        ]]
      ]]
    ]]
  ];
  l
]
Place[DG, places_List] := {Diagram[DG[1]]}
Place[DFF, places_List] := Module[
  {l={}, n=Length[places], t1, t2},
  Flatten[Table[
    Diagram[DFF @@ places[[{t1,t2}]]],
    {t1, n-1}, {t2, t1+1, n}
  ]]
]
Place[DFS, places_List] := Diagram /@ Module[
  {l={}, n=Length[places], t1, t2},
  For[t1=1, t1<=n, ++t1,
    For[t2=1, t2<=n, ++t2,
      If[t2!=t1, AppendTo[l, DFS @@ places[[{t1,t2}]]]]
    ]
  ];
  l
]
Place[DF4T, places_List] := Diagram /@ Module[
  {l={}, n=Length[places], t1, t2, t3, t4},
  For[t1=1, t1<=n, ++t1,
    For[t2=1, t2<n, ++t2,
      If[t2!=t1, For[t3=t2+1, t3<=n, ++t3,
        If[t3!=t1, For[t4=1, t4<=n, ++t4,
          If[!MemberQ[{t1,t2,t3}, t4],
            AppendTo[l, DF4T @@ places[[{t1,t2,t3,t4}]]]
          ]
        ]]
      ]]
    ]
  ];
  l
]

Place[specs_Plus, places_List] := Flatten[
  Function[place,
    Join[#, place]& /@ Place[
      Rest[specs],
      Complement[places, Join @@ Apply[List, place, {1}]]
    ]
  ] /@ Place[First[specs], places]
]
Place[specs_] := Union[CF[Place[specs, Range[Legs[specs]]]]]

Kbar[n_] := Union[CF[Place[n*Chord]]]
K1bar[n_] := Union[CF[
  Place[D4T+(n-2)*Chord, Range[2n-1]]
]]
K2bar[n_, rels_List] := Union[CF[Union @@ (
  If[n>=Deg[#],
    Place[# + (n-Deg[#])*Chord],
    {}
  ]& /@ rels
)]]
K2bar[n_] := K2bar[n, $DefaultRelations]

Kubar[n_] := Union[CF[Place[n*Chord, Range[2n]]]]
Ku1bar[n_] := Union[
  CF[Place[D4T+(n-2)*Chord, Range[2n-1]]],
  CF[Place[DF+(n-1)*Chord, Range[2n-1]]]
]
Ku2bar[n_, rels_List] := Union[CF[Union @@ (
  If[n>=Deg[#],
    Place[#+(n-Deg[#])*Chord],
    {}
  ]& /@ rels
)]]
Ku2bar[n_] := K2bar[n, $DefaultRelations~Join~{DFF, DFS, DF4T}]

KAbar[n_] := Union[CF[Place[n*Arrow, Range[2n]]]]
KA1bar[n_] := Union[CF[
  Place[D6T+(n-2)*Arrow, Range[2n-1]]
]]

KAubar[n_] := Union[CF[Place[n*Arrow, Range[2n]]]]
KAu1bar[n_] := CF[Union[
  Place[D6T+(n-2)*Arrow, Range[2n-1]],
  Place[DFL+(n-1)*Arrow, Range[2n-1]],
  Place[DFR+(n-1)*Arrow, Range[2n-1]]
]]

Lbar[n_, c_] := Union[Place[(c-1)*DCut+n*Chord, Range[2n+c-1]]]
L1bar[n_, c_] := Union[Place[D4T+(c-1)*DCut+(n-2)*Chord, Range[2n+c-2]]]
L2bar[n_, c_, rels_List] := Union @@ (
  If[ n>=Deg[#], Place[#+(c-1)*DCut+(n-Deg[#])*Chord], {}]& /@ rels
)
L2bar[n_, c_] := L2bar[n, c, $DefaultRelations]

NewBefore[i_, d_Diagram] := (d /. j_ :> j+1 /; j>=i) /. New -> i
NewAfter[i_, d_Diagram] := (d /. j_ :> j+1 /; j>i) /. New -> i+1

b[expr_] := Expand[CF[expr /. {
  Diagram[D4T[i_, j_, k_], chords___] :> (
    +NewAfter[k, Diagram[Chord[j,k], Chord[i, New], chords]]
    -NewBefore[k, Diagram[Chord[j,k], Chord[i, New], chords]]
    +NewAfter[j, Diagram[Chord[j,k], Chord[i, New], chords]]
    -NewBefore[j, Diagram[Chord[j,k], Chord[i, New], chords]]
  ),
  Diagram[D6T[i_, j_, k_], chords___] :> (
    +NewAfter[j, Diagram[Arrow[i,j], Arrow[New, k], chords]]
    -NewBefore[j, Diagram[Arrow[i,j], Arrow[New, k], chords]]
    -NewAfter[i, Diagram[Arrow[i,k], Arrow[New, j], chords]]
    +NewBefore[i, Diagram[Arrow[i,k], Arrow[New, j], chords]]
    -NewAfter[k, Diagram[Arrow[i, New], Arrow[j, k], chords]]
    +NewBefore[k, Diagram[Arrow[i, New], Arrow[j, k], chords]]
  ),
  Diagram[DF[i_], chords___] :>
    NewAfter[i, Diagram[Chord[i,New], chords]],
  Diagram[DFL[i_], chords___] :>
    NewBefore[i, Diagram[Arrow[i,New], chords]],
  Diagram[DFR[i_], chords___] :>
    NewAfter[i, Diagram[Arrow[i,New], chords]],
  Diagram[D1T[i_], chords___] :> ReplacePart[
    Diagram[D1T[i], chords] /. j_ :> j+2 /; j>i, D4T[i+1, i, i+2], 1
  ],
  Diagram[D2T[i_], chords___] :> (
    +Join[
      Diagram[D4T[i+2, i, i+3], Chord[i+1, i+4]],
      Diagram[chords] /. j_ :> j+4 /; j>i
    ]
    +Join[
      Diagram[D4T[i+2, i+1, i+4], Chord[i, i+3]],
      Diagram[chords] /. j_ :> j+4 /; j>i
    ]
  ),
  Diagram[D3T[i_, j_, k_], chords___] :> (
    +Diagram[D4T[i, j, k], chords]
    +Diagram[D4T[j, k, i], chords]
    +Diagram[D4T[k, i, j], chords]
  ),
  Diagram[D8T1[i1_, j1_, k1_, i2_, j2_, k2_], chords___] :> (
    +NewAfter[k1, Diagram[D4T[i2, j2, k2], Chord[j1, k1], Chord[i1, New], chords]]
    -NewBefore[k1, Diagram[D4T[i2, j2, k2], Chord[j1, k1], Chord[i1, New], chords]]
    +NewAfter[j1, Diagram[D4T[i2, j2, k2], Chord[j1, k1], Chord[i1, New], chords]]
    -NewBefore[j1, Diagram[D4T[i2, j2, k2], Chord[j1, k1], Chord[i1, New], chords]]
    -NewAfter[k2, Diagram[D4T[i1, j1, k1], Chord[j2, k2], Chord[i2, New], chords]]
    +NewBefore[k2, Diagram[D4T[i1, j1, k1], Chord[j2, k2], Chord[i2, New], chords]]
    -NewAfter[j2, Diagram[D4T[i1, j1, k1], Chord[j2, k2], Chord[i2, New], chords]]
    +NewBefore[j2, Diagram[D4T[i1, j1, k1], Chord[j2, k2], Chord[i2, New], chords]]
  ),
  Diagram[D8T2[i_, j_, k_, l_], chords___] :> (
    +NewAfter[j, Diagram[D4T[New, k, l], Chord[i, j], chords]]
    -NewBefore[j, Diagram[D4T[New, k, l], Chord[i, j], chords]]
    +NewAfter[i, Diagram[D4T[New, k, l], Chord[i, j], chords]]
    -NewBefore[i, Diagram[D4T[New, k, l], Chord[i, j], chords]]
    -NewAfter[l, Diagram[D4T[New, i, j], Chord[k, l], chords]]
    +NewBefore[l, Diagram[D4T[New, i, j], Chord[k, l], chords]]
    -NewAfter[k, Diagram[D4T[New, i, j], Chord[k, l], chords]]
    +NewBefore[k, Diagram[D4T[New, i, j], Chord[k, l], chords]]
  ),
  Diagram[D14T[i_, j_, k_, l_], chords___] :> (
    +NewAfter[l, Diagram[D4T[j,k,l], Chord[i, New], chords]]
    -NewBefore[l, Diagram[D4T[j,k,l], Chord[i, New], chords]]
    +NewAfter[k, Diagram[D4T[j,k,l], Chord[i, New], chords]]
    -NewBefore[k, Diagram[D4T[j,k,l], Chord[i, New], chords]]
    +NewAfter[j, Diagram[D4T[j,k,l], Chord[i, New], chords]]
    -NewBefore[j, Diagram[D4T[j,k,l], Chord[i, New], chords]]
    -NewAfter[l, Diagram[D4T[i,k,l], Chord[j, New], chords]]
    +NewBefore[l, Diagram[D4T[i,k,l], Chord[j, New], chords]]
    -NewAfter[k, Diagram[D4T[i,k,l], Chord[j, New], chords]]
    +NewBefore[k, Diagram[D4T[i,k,l], Chord[j, New], chords]]
    -NewAfter[l, Diagram[D4T[i,j,New], Chord[k,l], chords]]
    +NewBefore[l, Diagram[D4T[i,j,New], Chord[k,l], chords]]
    -NewAfter[k, Diagram[D4T[i,j,New], Chord[k,l], chords]]
    +NewBefore[k, Diagram[D4T[i,j,New], Chord[k,l], chords]]
  ),
  Diagram[DG[1], chords___] :> Sum[
    Prepend[
      Delete[Diagram[chords], i],
      Prepend[D4T @@ {chords}[[i]], 1]
    ],
    {i,Length[{chords}]}
  ],
  Diagram[DFF[i_, j_], chords___] :>
   +NewAfter[i, Diagram[DF[j], Chord[i, New], chords]]
   -NewAfter[j, Diagram[DF[i], Chord[j, New], chords]],
  Diagram[DFS[i_, j_], chords___] :>
   +NewAfter[i, Diagram[DF[New], Chord[i, j], chords]]
   -NewBefore[i, Diagram[DF[New], Chord[i, j], chords]]
   +NewAfter[i, Diagram[D4T[j, i, New], chords]],
  Diagram[DF4T[i_, j_, k_, l_], chords___] :> (
    +NewAfter[k, Diagram[Chord[j,k], Chord[i, New], DF[l], chords]]
    -NewBefore[k, Diagram[Chord[j,k], Chord[i, New], DF[l], chords]]
    +NewAfter[j, Diagram[Chord[j,k], Chord[i, New], DF[l], chords]]
    -NewBefore[j, Diagram[Chord[j,k], Chord[i, New], DF[l], chords]]
    -NewAfter[l, Diagram[D4T[i,j,k], Chord[l, New], chords]]
  )
}]]

alpha[x_] := Expand[x /. (
  d_Diagram :> CF[Distribute[d /. Chord[i_,j_] :> Arrow[i,j] + Arrow[j,i]]]
)]

beta[x_] := Module[
  {s,t},
  Expand[
    (x /.  d_Diagram :> Distribute[d /. Chord[i_,j_] :> Arrow[i,j] - Arrow[j,i]])
      /. t_Diagram :> (-1)^Count[t, -1, Infinity] * CF[t /. -1 -> 1]
  ]
]

CutAfter[i_Integer, expr_] := expr /. d_Diagram :>
  NewAfter[i, Prepend[d, DCut[New]]]

Chalpha[n_] := (
  K0 = Kbar[n];
  Print["dim K0 = ", Length[K0]];
  K1 = K1bar[n];
  Print["dim K1 = ", Length[K1]];
  rel = NewSubspace[K0];
  AddVector[rel, b[#]]& /@ K1;
  Print["dim rel = ", dimrel=SubspaceDimension[rel]];
  Print["dim H0 = ", Length[K0]-dimrel];
  KA0 = KAbar[n];
  Print["dim KA0 = ", Length[KA0]];
  KA1 = KA1bar[n];
  Print["dim KA1 = ", Length[KA1]];
  relA = NewSubspace[KA0];
  AddVector[relA, b[#]]& /@ KA1;
  Print["dim relA = ", dimrelA=SubspaceDimension[relA]];
  Print["dim HA0 = ", Length[KA0]-dimrelA];
  AddVector[relA, alpha[b[#]]]& /@ K1;
  If[dimrelA == SubspaceDimension[relA], (
    im = NewSubspace[KA0];
    AddVector[im, #]& /@ SubspaceBasis[relA];
    AddVector[im, alpha[#]]& /@ K0;
    Print["dim im = ", dimim = SubspaceDimension[im]];
    Print["dim ker = ", dimim-dimrelA-Length[K0]+dimrel];
  ),
    Print["alpha not well defined!! New dim relA = ", SubspaceDimension[relA]]
  ]
)

DimH1[n_] := (
  K1 = K1bar[n];
  Print["dim K1 = ", Length[K1]];
  K2 = K2bar[n];
  Print["dim K2 = ", Length[K2]];
  ker = LinearDependencies[b[K1]] . K1;
  Print["dim ker = ", Length[ker]];
  im = LinearSpan[b[K2]];
  Print["dim im = ", Length[im]];
  Length[ker]-Length[im]
)

Hu1[n_] := (
  Ku0 = Kubar[n];
  Print["dim Ku0 = ", Length[Ku0]];
  Ku1 = Ku1bar[n];
  Print["dim Ku1 = ", Length[Ku1]];
  Ku2 = Ku2bar[n];
  Print["dim Ku2 = ", Length[Ku2]];
  ker = LinearDependencies[b[Ku1]] . Ku1;
  Print["dim ker = ", Length[ker]];
  im = LinearSpan[b[Ku2]];
  Print["dim im = ", Length[im]];
  Length[ker]-Length[im]
)

LH1[n_, c_] := (
  L1 = L1bar[n, c];
  Print["dim L1 = ", Length[L1]];
  L2 = L2bar[n, c];
  Print["dim L2 = ", Length[L2]];
  ker = LinearDependencies[b[L1]] . L1;
  Print["dim ker = ", Length[ker]];
  im = LinearSpan[b[L2]];
  Print["dim im = ", Length[im]];
  Length[ker]-Length[im]
)

LH0[n_, c_] := (
  L0 = Lbar[n, c];
  Print["dim L0 = ", Length[L0]];
  L1 = L1bar[n, c];
  Print["dim L1 = ", Length[L1]];
  im = LinearSpan[b[L1]];
  Print["dim im = ", Length[im]];
  Length[L0]-Length[im]
)

HA0[n_] := (
  K0 = KAbar[n];
  Print["dim KA0 = ", Length[K0]];
  K1 = KA1bar[n];
  Print["dim KA1 = ", Length[K1]];
  im = LinearSpan[b[K1]];
  Print["dim im = ", Length[im]];
  Length[K0]-Length[im]
)

HAu0[n_] := (
  K0 = KAubar[n];
  Print["dim KAu0 = ", Length[K0]];
  K1 = KAu1bar[n];
  Print["dim KAu1 = ", Length[K1]];
  im = LinearSpan[b[K1]];
  Print["dim im = ", Length[im]];
  Length[K0]-Length[im]
)

FastKH1[n_] := (
  K0 = Kbar[n];
  Print["dim K0 = ", Length[K0]];
  K1 = K1bar[n];
  Print["dim K1 = ", Length[K1]];
  K2 = K2bar[n];
  Print["dim K2 = ", Length[K2]];
  im1 = NewSubspace[K0];
  AddVector[im1, PrintPass[b[#]]]& /@ K1;
  Print["dim im1 = ", SubspaceDimension[im1]];
  im2 = NewSubspace[K1];
  AddVector[im2, PrintPass[b[#]]]& /@ K2;
  Print["dim im2 = ", SubspaceDimension[im2]];
  Length[K1]-SubspaceDimension[im1]-SubspaceDimension[im2]
)

FastLH1[n_, c_] := (
  L0 = Lbar[n, c];
  Print["dim L0 = ", Length[L0]];
  L1 = L1bar[n, c];
  Print["dim L1 = ", Length[L1]];
  L2 = L2bar[n, c];
  Print["dim L2 = ", Length[L2]];
  im1 = NewSubspace[L0];
  AddVector[im1, b[#]]& /@ L1;
  Print["dim im1 = ", SubspaceDimension[im1]];
  im2 = NewSubspace[L1];
  AddVector[im2, b[#]]& /@ L2;
  Print["dim im2 = ", SubspaceDimension[im2]];
  Length[L1]-SubspaceDimension[im1]-SubspaceDimension[im2]
)

PullLeft[n_] := (
  +Sum[
    Diagram @@ Join[
      {D4T[2n-i, i, i+1]},
      Table[Chord[j, 2n-j], {j,i-1}],
      Table[Chord[j+1, 2n-j] ,{j,i+1,n-1}]
    ],
    {i,n-1}
  ]
  +Sum[
    Diagram @@ Join[
      {D4T[1, i+1, 2n-i]},
      Table[Chord[j+1, 2n-j], {j,i-1}],
      Table[Chord[j+1, 2n-j] ,{j,i+1,n-1}]
    ],
    {i,n-1}
  ]
)

StarRelation[n_] := Sum[
  Diagram @@ Join[
    {D4T[n, i, n+i]},
    Table[Chord[j, n+j], {j,i-1}],
    Table[Chord[j, n+j], {j,i+1, n-1}]
  ],
  {i,n-1}
]

Adjoint[A_List, B_List, d_List] := Table[
  Inner[(Coefficient[#1, B[[i]]] * #2)&, d, A, Plus],
  {i, Length[B]}
]

CD2Diagram[cd_CD] := Diagram @@ Table[
  Chord @@ Flatten[Position[cd, i]],
  {i, Length[cd]/2}
]

TeXFormat[d_Diagram] := Module[
  {
    l=Plus @@ (Length /@ d),
    dd=d,
    pi=N[Pi],
    n,i,j,k,x,y,z
  },
  n[z_] := N[Round[100000*z]/100000];
  x[i_] := n[Cos[2pi*i/l]];
  y[i_] := n[Sin[2pi*i/l]];
  If[l>2, Do[
    If[MatchQ[dd[[k]], Chord[i_,j_] /; j==i+1],
      ++l; dd = dd /. j_Integer :> j+1 /; j>dd[[k,1]]
     ];
    If[MatchQ[dd[[k]], D4T[i_,j_,k_] /; k==j+1],
      ++l; dd = dd /. j_Integer :> j+1 /; j>dd[[k,2]]
     ],
    {k,Length[d]}
  ]];
  StringJoin[
    "\parbox{4mm}{\setlength{\unitlength}{2mm} ",
    "\begin{picture}(2,2)(-1,-1) \put(0,0){\bigcircle{2}} ",
    StringJoin @@ (
      dd /. {
        Chord[i_,j_] :> ToString[StringForm[
          "\curve(``,``,``,``) ",
          x[i], y[i], x[j], y[j]
        ]],
        D4T[i_,j_,k_] :> ToString[StringForm[
          "\curve(``,``,``,``) \curve(``,``,``,``) ",
          x[j], y[j], x[k], y[k], x[i], y[i], (x[j]+x[k])/2, (y[j]+y[k])/2
        ]]
      }
    ),
    "\end{picture}} "
  ]
]
Format[d_Diagram, TeXForm] := TeXFormat[d]
